import * as CANNON from 'cannon-es';
import { DRACOLoader } from 'three/examples/jsm/loaders/DRACOLoader';
import { FirstPersonControls } from 'three/examples/jsm/controls/FirstPersonControls.js';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls.js';
import { PointerLockControls } from 'three/examples/jsm/controls/PointerLockControls.js';
import * as THREE_2 from 'three';
import { TransformControls } from 'three/examples/jsm/controls/TransformControls';

export declare class _AnimateCSS {
    _animation_base_class: string;
    _animations: {
        BOUNCE: string;
        FLASH: string;
        PULSE: string;
        RUBBER_BAND: string;
        SHAKE_X: string;
        SHAKE_Y: string;
        HEAD_SHAKE: string;
        SWING: string;
        TADA: string;
        WOBBLE: string;
        JELLO: string;
        HEART_BEAT: string;
        BACK_IN_DOWN: string;
        BACK_IN_LEFT: string;
        BACK_IN_RIGHT: string;
        BACK_IN_UP: string;
        BACK_OUT_DOWN: string;
        BACK_OUT_LEFT: string;
        BACK_OUT_RIGHT: string;
        BACK_OUT_UP: string;
        BOUNCE_IN: string;
        BOUNCE_IN_DOWN: string;
        BOUNCE_IN_LEFT: string;
        BOUNCE_IN_RIGHT: string;
        BOUNCE_IN_UP: string;
        BOUNCE_OUT: string;
        BOUNCE_OUT_DOWN: string;
        BOUNCE_OUT_LEFT: string;
        BOUNCE_OUT_RIGHT: string;
        BOUNCE_OUT_UP: string;
        FADE_IN: string;
        FADE_IN_DOWN: string;
        FADE_IN_DOWN_BIG: string;
        FADE_IN_LEFT: string;
        FADE_IN_LEFT_BIG: string;
        FADE_IN_RIGHT: string;
        FADE_IN_RIGHT_BIG: string;
        FADE_IN_UP: string;
        FADE_IN_UP_BIG: string;
        FADE_IN_TOP_LEFT: string;
        FADE_IN_TOP_RIGHT: string;
        FADE_IN_BOTTOM_LEFT: string;
        FADE_IN_BOTTOM_RIGHT: string;
        FADE_OUT: string;
        FADE_OUT_DOWN: string;
        FADE_OUT_DOWN_BIG: string;
        FADE_OUT_LEFT: string;
        FADE_OUT_LEFT_BIG: string;
        FADE_OUT_RIGHT: string;
        FADE_OUT_RIGHT_BIG: string;
        FADE_OUT_UP: string;
        FADE_OUT_UP_BIG: string;
        FADE_OUT_TOP_LEFT: string;
        FADE_OUT_TOP_RIGHT: string;
        FADE_OUT_BOTTOM_RIGHT: string;
        FADE_OUT_BOTTOM_LEFT: string;
        FLIP: string;
        FLIP_IN_X: string;
        FLIP_IN_Y: string;
        FLIP_OUT_X: string;
        FLIP_OUT_Y: string;
        LIGHT_SPEED_IN_RIGHT: string;
        LIGHT_SPEED_IN_LEFT: string;
        LIGHT_SPEED_OUT_RIGHT: string;
        LIGHT_SPEED_OUT_LEFT: string;
        ROTATE_IN: string;
        ROTATE_IN_DOWN_LEFT: string;
        ROTATE_IN_DOWN_RIGHT: string;
        ROTATE_IN_UP_LEFT: string;
        ROTATE_IN_UP_RIGHT: string;
        ROTATE_OUT: string;
        ROTATE_OUT_DOWN_LEFT: string;
        ROTATE_OUT_DOWN_RIGHT: string;
        ROTATE_OUT_UP_LEFT: string;
        ROTATE_OUT_UP_RIGHT: string;
        HINGE: string;
        JACK_IN_THE_BOX: string;
        ROLL_IN: string;
        ROLL_OUT: string;
        ZOOM_IN: string;
        ZOOM_IN_DOWN: string;
        ZOOM_IN_LEFT: string;
        ZOOM_IN_RIGHT: string;
        ZOOM_IN_UP: string;
        ZOOM_OUT: string;
        ZOOM_OUT_DOWN: string;
        ZOOM_OUT_LEFT: string;
        ZOOM_OUT_RIGHT: string;
        ZOOM_OUT_UP: string;
        SLIDE_IN_DOWN: string;
        SLIDE_IN_LEFT: string;
        SLIDE_IN_RIGHT: string;
        SLIDE_IN_UP: string;
        SLIDE_OUT_DOWN: string;
        SLIDE_OUT_LEFT: string;
        SLIDE_OUT_RIGHT: string;
        SLIDE_OUT_UP: string;
    };
}

export declare type HTMLEventListenersIndex = {
    [id: string]: {
        _listener: Function;
        _event_name: string;
        _object?: any;
    };
};

/**
 * @interface IX3DObjectData
 */
export declare interface IX3DObjectData extends XObjectData {
    [key: string]: any;
    _cannon_shape?: CANNON.Shape | undefined;
    _enable_physics?: boolean;
    _mass?: number;
    _position?: XVector3Data;
    _rotation?: XVector3Data;
    _scale?: XVector3Data;
    _visible?: boolean;
    _fade_duration?: number;
    _three_class?: any;
    _threes_class_args?: Array<any>;
    _disable_frame_3d_state?: boolean;
    _3d_set_once?: boolean;
    _positional_audio_source?: string;
    _load_animations?: boolean;
    _auto_play_animation?: boolean;
    _model_url?: string;
    _on_load?: X3DListener;
    _on_click?: X3DListener;
}

declare interface IXData {
    [k: string]: string | null | [] | undefined | Function | boolean | number | {};
}

declare interface IXData_2 {
    [k: string]: string | null | [] | undefined | Function | boolean | number | {};
}

/**
 * XObject constructor data interface
 * @interface IXObjectData
 * @param _xversion - minimum Xpell interpreter version (optional default value is 1.0)
 * @deprecated use XObjectData type instead instead
 */
export declare interface IXObjectData extends IXData {
    [k: string]: string | null | [] | undefined | Function | boolean | number | {};
    _id?: string;
    _type: string;
    _children?: Array<XObject | XObjectData>;
    _name?: string;
    _data_source?: string;
    _on?: XObjectOnEventIndex;
    _on_create?: string | Function | undefined;
    _on_mount?: string | Function | undefined;
    _on_frame?: string | Function | undefined;
    _on_data?: string | Function | undefined;
    _process_frame?: boolean;
    _process_data?: boolean;
}

export declare enum MessageType {
    Text = 0,
    JSON = 1
}

declare interface WaitersPack {
    [waiterID: string]: CallableFunction;
}

export declare const WormholeEvents: {
    WormholeOpen: string;
    WormholeClose: string;
    ResponseDataArrived: string;
};

declare class WormholeInstance {
    _ws: null | WebSocket;
    _ready: boolean;
    _data_waiters: WaitersPack;
    _log_rules: {
        [key: string]: boolean;
    };
    _on_open?: CallableFunction;
    constructor();
    /**
     * Set the log rules for the wormhole
     */
    set verbose(val: boolean);
    private createMessage;
    /**
     * Opens a Wormhole to the PAI-BOT on the server (client-to-server wormhole)
     * @param {*} url
     */
    open(url: string): void;
    close(): void;
    /**
     * sends the message to the server (asynchronous) and return the result in a callback function
     * @param message - the message to send
     * @param cb - Callback Function (data: string) => void
     * @param type - type of the message MessageType.JSON / MessageType.Text
     */
    send(message: any, cb: CallableFunction, type?: MessageType): void;
    /**
     * Sends the message to the server and return the result as a response (synchronous)
     * @param message - the message to send
     * @param checkXProtocol - check the result for XProtocol (True by default, if false the response will be return as is)
     */
    sendSync(message: any, checkXProtocol?: boolean): Promise<WormholeMessage | any>;
    private stringify;
    onOpen(cb: CallableFunction): void;
}

declare interface WormholeMessage {
    id: string;
    type: MessageType;
    data: string;
}

declare const Wormholes: WormholeInstance;
export { Wormholes }
export { Wormholes as _wh }

/**
 * X3DHelper Data
 */
export declare type X3AxesHelper = {
    size: number;
};

export declare const X3D: X3DModule;

export declare type X3DApp = {
    _parent_element: string;
    _physics: {
        _engine?: X3DPhysicsEngines;
        _active: boolean;
        _debug?: boolean;
    };
    _scene: {
        _helpers?: {
            [k: string]: XHelperData<X3AxesHelper>;
        };
        _lights?: {
            [k: string]: XLightData;
        };
        _cameras?: {
            [k: string]: XCameraData;
        };
        _background?: X3DSceneBackground;
        _controls?: {
            [k: string]: X3DSceneControl;
        };
        _objects?: {
            [k: string]: IX3DObjectData;
        };
        _raycast?: boolean;
    };
};

export declare class X3DAppGenerator {
    static getDefaultApp(orbitControls?: boolean, bgColor?: string, enablePhysics?: boolean): X3DApp;
}

export declare const X3DDefaultApp: X3DApp;

export declare type X3DHelpers = "axes" | "skeleton";

export declare type X3DListener = (x3dObject: X3DObject, event?: any) => void;

export declare const X3DLoader: _X3DLoader;

declare class _X3DLoader {
    loading: boolean;
    dracoLoader: DRACOLoader;
    constructor();
    loadDraco(): void;
    loadFBXAnimation(fileName: string, x3dObject: X3DObject, onLoadCallBack?: Function): void;
    loadModelFromGLTF(modelUrl: string, traverse?: boolean): Promise<THREE_2.Object3D>;
}

/**
 * @class X3DModule - Xpell 3D Module
 */
export declare class X3DModule extends XModule {
    world: X3DWorld;
    x3dObjects: {};
    status: number;
    _raycast_event: string;
    constructor();
    /**
     * This method creates a player element and append it to the DOM
     * @param playerId - optional id of the player element
     * @param parentElementId - optional id of the element to append the player to, if not provided the player will be appended to the body
     * @returns HTMLDivElement
     */
    createPlayer(playerId?: string, cssClass?: string, parentElementId?: string): HTMLDivElement;
    /**
     * Loads XWorld from world data object
     * @param x3dWorldData World Data object
     * @param autoRun - start world running automatically
     * @deprecated - use loadApp function instead
     */
    loadWorld(x3dWorldData: X3DApp, autoRun?: boolean): Promise<void>;
    loadDefaultApp(orbitControls?: boolean, bgColor?: string, enablePhysics?: boolean): Promise<X3DApp>;
    loadApp(x3dApp: X3DApp, autoRun?: boolean): Promise<void>;
    /**
     * Creates new X3DObject from data object
     * @param data - The data of the new object (JSON)
     * @return {X3DObject}
     * @override
     */
    create(data: IX3DObjectData): Promise<any>;
    /**
     * Removes X3DObject from X3D engine (including world and X3D object manager)
     * @param objectId th e_id of the X3D object to remove
     */
    remove(objectId: string): Promise<void>;
    /**
     * Add X3DObject to the object manager and world
     *
     * @param x3dObject X3DObject | IX3DObjectData - The X3DObject to add
     * @returns The added X3DObject (after creation)
     */
    add(x3dObject: X3DObject | IX3DObjectData): Promise<X3DObject>;
    /**
     * Add X3DObject to the object manager and world
     * @param x3dObject - The X3DObject to add
     *
     */
    loadObject(data: IX3DObjectData): Promise<any>;
    /**
     * Creates new X3DObject from data and add it to the world
     * @param <IX3DObjectData> data  - the object to create data (json)
     * @returns
     */
    addRaw(data: IX3DObjectData): Promise<X3DObject>;
    onWindowResize(): void;
    start(): Promise<void>;
    enableRaycast(event?: string): void;
    disableRaycast(): void;
    raycast(e: any): void;
    /**
     * Add sky map (background to the world scene)
     * @param path - path to skymap
     * @param images - optional images array like ["px.jpg","nx.jpg","py.jpg","ny.jpg","pz.jpg","nz.jpg"]
     */
    addEnvironmentMap(path: string, images?: string[]): void;
    onFrame(frameNumber: number): Promise<void>;
}

export declare class X3DObject extends XObject {
    #private;
    _id: string;
    _type: string;
    _name: string;
    _three_class: any;
    _threes_class_args: Array<any>;
    _three_obj: THREE_2.Object3D | null;
    _cannon_obj: CANNON.Body | undefined | null;
    _cannon_shape: CANNON.Shape | undefined | null;
    _mass: number;
    _enable_physics: boolean;
    _position: THREE_2.Vector3;
    _rotation: THREE_2.Euler;
    _scale: THREE_2.Vector3;
    _visible: boolean;
    _model_url: string;
    _positional_audio_source: string;
    _3d_set_once: boolean;
    _fade_duration: number;
    _disable_frame_3d_state: boolean;
    _load_animations: boolean;
    _auto_play_animation: boolean;
    _on_load?: X3DListener | undefined;
    _on_click?: X3DListener | undefined;
    /**
     * protected fields should not be override by XData object
     */
    protected _animation_clips: {
        [name: string]: THREE_2.AnimationAction;
    };
    protected _clock: THREE_2.Clock;
    protected _fraction: number;
    protected _animation_mixer: THREE_2.AnimationMixer | null;
    protected _current_action: string | null;
    protected _positional_audio: THREE_2.PositionalAudio | undefined | null;
    protected _xem_options: XEventListenerOptions;
    protected _log_rules: {
        _import_animation: boolean;
        _play_animation: boolean;
        _load_model: boolean;
    };
    static getXData(threeObj: THREE_2.Object3D, defaults: IX3DObjectData): {
        _id: string;
        _type: string;
        _children: never[];
        _three_obj: THREE_2.Object3D<THREE_2.Event>;
        _name: string;
        _position: THREE_2.Vector3;
        _rotation: THREE_2.Euler;
        _scale: THREE_2.Vector3;
        _enable_physics: boolean;
    };
    static getFromThreeObject(threeObj: THREE_2.Object3D, defaults: IX3DObjectData): X3DObject;
    constructor(data: IX3DObjectData, defaults?: any, skipParse?: boolean);
    parse(data: XObjectData, init?: boolean): void;
    init(): void;
    /**
     * Dispose all object memory (destructor)
     */
    dispose(): Promise<void>;
    /**
     * This method parses the X3DObject 3D data and sets the X3DObject properties
     * @param data
     */
    parse3d(data: IX3DObjectData): void;
    /**
     * Sets the X3DObject position
     * @param positionObject the new position X3DObject in the form of {x:number,y:number,z:number}
     * @example setPosition({x:0,y:0,z:0})
     * @comment effects only if Xpell engine controls the position (_disable_frame_3d_state = false - default)
     */
    setPosition(positionObject: {
        x: number;
        y: number;
        z: number;
    }): void;
    /**
     * Sets the X3DObject position from a THREE.Vector3 object
     * @param newPosition the new position X3DObject in the form of THREE.Vector3
     */
    setPositionFromVector3(newPosition: THREE_2.Vector3): void;
    /**
     * Sets the X3DObject rotation
     * @param rotationObject - The new rotation X3DObject in the form of {x:number,y:number,z:number,order?:string}
     * @example setRotation({x:0,y:0,z:0,order:"XYZ"})
     * @comment effects only if Xpell engine controls the position (_disable_frame_3d_state = false - default)
     */
    setRotation(rotationObject: {
        x: number;
        y: number;
        z: number;
        order?: string;
    }): void;
    /**
     * Sets the X3DObject rotation from a THREE.Euler object
     * @param newRotation  The new rotation X3DObject in the form of THREE.Euler
     */
    setRotationFromEuler(newRotation: THREE_2.Euler): void;
    /**
     * Sets the X3DObject rotation from a THREE.Quaternion object
     * @param newQuaternion The new rotation X3DObject in the form of THREE.Quaternion
     */
    setRotationFromQuaternion(newQuaternion: THREE_2.Quaternion): void;
    /**
     * Sets the X3DObject scale
     * @param newScale The new scale X3DObject in the form of {x:number,y:number,z:number}
     * @example setScale({x:1,y:1,z:1})
     * @comment effects only if Xpell engine controls the position (_disable_frame_3d_state = false - default)
     */
    setScale(newScale: {
        x: number;
        y: number;
        z: number;
    }): void;
    /**
     * Sets the X3DObject scale from a THREE.Vector3 object
     * @param newScale The new scale X3DObject in the form of THREE.Vector3
     */
    setScaleFromVector3(newScale: THREE_2.Vector3): void;
    /**
     * This method sets the 3D State of the object (position, rotation & scale).
     */
    set3DState(): void;
    /**
     * This method gets the Three object of the X3DObject
     */
    getThreeObject(): THREE_2.Object3D | Promise<THREE_2.Object3D>;
    /**
     * This method returns the Canon physic's object of the X3DObject
     * @returns CANNON.Body
     * @comment if the X3DObject physics should be enabled the
     *          _enable_physics attribute should be set to true
     *          and the _enable_physics attribute for the main engine should be set to true
     */
    getCannonObject(): CANNON.Body;
    get canon(): CANNON.Body;
    get three(): THREE_2.Object3D<THREE_2.Event> | Promise<THREE_2.Object3D<THREE_2.Event>>;
    /**
     * This method creates Positional Audio from a source file and attach it to the 3D object
     * @param source - the source file path (mp3, wav, ogg...)
     * @param data - optional data object that can contain "autoplay" and "loop" boolean values
     * @returns
     */
    createPositionalAudio(source: string, data?: IX3DObjectData): Promise<THREE_2.PositionalAudio>;
    setPositionalAudioSource(source?: string, data?: IX3DObjectData): Promise<void>;
    /**
     * clears the positional audio from the object and the Three object
     */
    clearPositionalAudio(): void;
    playAudio(loop?: boolean): void;
    pauseAudio(): void;
    /**
     * onFrame function for x3d-object
     * - parse textual command to SpellCommand and cache
     * - set 3d-state (position, rotation & scale) if Spell in control
     * - update animation mixer if exists
     * @param {number} frameNumber
     */
    onFrame(frameNumber: number): Promise<void>;
    /**
     * Append X3DObject as a child object
     * @param x3dObject
     */
    append(x3dObject: any | IX3DObjectData): Promise<void>;
    /**
     * Show the X3DObject (if it was hidden)
     */
    show(): void;
    /**
     * Hide the X3DObject
     */
    hide(): void;
    /**
     * Toggle the X3DObject visibility
     */
    toggle(): void;
    /**
     * Import animation from a THREE Object3D to the current object
     * @param threeObj ThreeJs Object3D to import the animations from
     * @param newName optional - change the animation name to a new name
     *                           (if there are more than one animation they
     *                            will be added with index: Idle, Idle-2, Idle -3 ...)
     */
    importAnimations(threeObj: THREE_2.Object3D, newName?: string): Promise<void>;
    /**
     * Loads a new 3D model to the X3DObject from a GLTF/GLB file
     * @param modelUrl - url of the model file
     * @returns Promise<THREE.Object3D>
     * @deprecated
     */
    loadThreeObjectFromGLTF(modelUrl: string): Promise<THREE_2.Object3D>;
    /**
     * Loads a 3d model from a GLTF/GLB file into the X3DObject and updates the Three object
     * @param modelUrl
     */
    loadModel(modelUrl?: string): Promise<void>;
    /**
     * This method is called after the 3D model is loaded
     * if override this method, call super.onLoad() to activate the default onLoad method
     */
    onLoad(): Promise<void>;
    onMount(): Promise<void>;
    /**
     * This method is called when the X3DObject is clicked
     * if override this method, call super.onClick() to activate the default onClick method
     */
    onClick(event?: any): Promise<void>;
    importAnimationFromGLTF(modelUrl: string, newName: string | undefined): Promise<void>;
    /**
     * Import animations from an FBX file (compatible with maximo.com animations)
     * @param url - url of the FBX file
     * @since 1.04
     */
    importAnimationFromFBXFile(url: string, newName?: string): Promise<void>;
    /**
     * loads animation on start or after create object
     */
    loadAnimations(): Promise<void>;
    stopAllAnimations(): void;
    playAllAnimations(): void;
    playAnimation(clipName: string, loop?: THREE_2.AnimationActionLoopStyles, repetitions?: number): void;
    stopAnimation(): void;
}

export declare type X3DPhysicsEngines = "cannon.js";

export declare class X3DPrimitives extends XObjectPack {
    static getObjects(): {
        material: typeof XMaterial;
        geometry: typeof XGeometry;
        mesh: typeof XMesh;
        group: typeof XGroup;
        plane: typeof XPlane;
        box: typeof XBox;
        sphere: typeof XSphere;
        cylinder: typeof XCylinder;
        torus: typeof XTorus;
        cone: typeof XCone;
        xmodel: typeof XModel;
        "perspective-camera": typeof XCamera;
        camera: typeof XCamera;
        light: typeof XLight;
        circle: typeof XCircle;
    };
}

/**
 * Background type
 * @description solid-color | gradient | image | video | cube-texture | sphere-texture
 * params for:
 * solid-color: color:string
 * gradient: color1, color2, direction
 * image: url
 * video: url
 * cube-texture: [url1, url2, url3, url4, url5, url6]
 * sphere-texture: url
 */
export declare type X3DSceneBackground = {
    _type: X3DSceneBackgroundTypes;
    _params: {
        [k: string]: any;
    };
};

export declare type X3DSceneBackgroundHandler = {
    (scene: THREE_2.Scene, params: X3DSceneBackgroundParams): void;
};

export declare type X3DSceneBackgroundParams = {
    [k: string]: any;
    _color?: string;
    _color1?: string;
    _color2?: string;
    _direction_deg?: number;
    _url?: string;
    _px?: string;
    _nx?: string;
    _py?: string;
    _ny?: string;
    _pz?: string;
    _nz?: string;
    _path?: string;
};

export declare enum X3DSceneBackgroundTypes {
    _solid_color = "solid-color",
    _gradient = "gradient",
    _image = "image",
    _video = "video",
    _cube_texture = "cube-texture",
    _sphere_texture = "sphere-texture",
    _hdri = "hdri"
}

export declare type X3DSceneControl = {
    _type: "orbit" | "pointer" | "first-person" | "transform";
    _active: boolean;
    _params?: {
        [k: string]: any;
    };
};

export declare class X3DWorld {
    status: number;
    worldRowData: X3DApp;
    scene: THREE_2.Scene;
    clock: THREE_2.Clock;
    renderer: THREE_2.WebGLRenderer;
    frameNumber: number;
    raycaster: THREE_2.Raycaster;
    transformControls: TransformControls;
    private transformControlX3dObject;
    private transformControlsListenerAdded;
    lights: {};
    x3dObjects: {
        [k: string]: X3DObject;
    };
    defaultCamera: any;
    controls: OrbitControls | PointerLockControls | FirstPersonControls;
    frameProcessTime: number;
    audioListener: THREE_2.AudioListener;
    enablePhysics: boolean;
    private _physics_world;
    _log_rules: {
        _add: boolean;
        _remove: boolean;
    };
    private _cannon_debug_renderer;
    private _bg_handlers;
    constructor(xworld: X3DApp);
    run(): Promise<void>;
    addBackgroundHandler(type: string, handler: X3DSceneBackgroundHandler): void;
    setSceneBackground(bg: X3DSceneBackground): Promise<void>;
    setAudioListener(): void;
    onWindowResize(): void;
    /**
     * Adds X3DObject to the world scene
     * @param {X3DObject}x3dObject - the X3DObject to add
     * @returns {X3DObject}X3DObject
     */
    addX3DObject(x3dObject: X3DObject): Promise<THREE_2.Object3D<THREE_2.Event> | undefined>;
    removeX3DObject(objectId: string): Promise<void>;
    removeTransformControls(): void;
    setTransformControls(x3dObject: X3DObject): void;
    onFrame(frameNumber: number): Promise<void>;
    /**
     * Adds background to the scene
     * @param {THREE.Texture}bgTexture scene background
     */
    addBackground(bgTexture: THREE_2.Texture): void;
    render(): void;
}

declare const XAI: XAIModule;
export { XAI }
export { XAI as _xai }

export declare class XAIModule extends XModule {
    /**
     * @fires "xai-loaded" event
     * @param data module data
     */
    constructor(data: XModuleData);
}

export declare class XAIObject extends XObject {
    _ignore: {
        [k: string]: string;
    };
    constructor(data: XObjectData, defaults: any);
}

export declare class XAIObjects extends XObjectPack {
    static getObjects(): {
        [x: string]: typeof XTTS;
    };
}

export declare class XBox extends XMesh {
    constructor(data: IX3DObjectData, defaults: any);
    set _width(val: number);
    get _width(): number | undefined;
    set _height(val: number);
    get _height(): number | undefined;
    set _depth(val: number);
    get _depth(): number | undefined;
}

export declare class XButton extends XUIObject {
    constructor(data: XObjectData);
    setOnclick(fun: CallableFunction): void;
}

/**
 * XCamera - Three Camera wrapper
 */
export declare class XCamera extends X3DObject {
    static xtype: string;
    readonly _is_camera: boolean;
    _fov: number;
    _ratio: number;
    _far: number;
    _close: number;
    _positional_audio_listener: boolean;
    _camera: XCameraTypes;
    constructor(data: XCameraData);
}

export declare interface XCameraData extends IX3DObjectData {
    _camera?: XCameraTypes;
    _fov?: number;
    _ratio?: number;
    _far?: number;
    _near?: number;
    _positional_audio_listener?: boolean;
    _helper?: boolean;
}

export declare type XCameraTypes = "perspective-camera" | "perspective" | "camera" | "orthographic-camera" | "orthographic";

export declare class XCircle extends XMesh {
    constructor(data: any, defaults: any);
}

/**
 * XCommand class - this command is being sent to the Xpell parser or every XModule/XObject for execution
 */
export declare class XCommand {
    /**
     * The XModule to handle to command
     */
    _module: string;
    /**
     * The XObject that should handle the command (optional - uses only to send XCommand to specific object)
     */
    _object?: string;
    /**
     * The command operation (op/method) to execute
     */
    _op: string;
    /**
     * command parameters array
     */
    _params?: {
        [k: string]: string | number | Function;
    };
    /**
     * XCommand create date timestamp
     */
    d: number;
    constructor(data?: XCommandData);
    /**
     * Gets th parameter value from the XCommand whether it has a name or just a position
     * There are 2 ways to send XCommand with parameters:
     *  1. <module> <op> <param-0> <param-1> <param-2>     // position is for this case
     *  2. <module> <op> param-name:param-value            // name is for this case
     * @param position the position of the parameter if no name is send
     * @param name the name of the parameter
     * @param defaultValue the default value if none above exists
     * @returns {any} the actual parameter value
     */
    getParam(position: number, name: string, defaultValue: any): any;
}

export declare type XCommandData = {
    _module: string;
    _object?: string;
    _op: string;
    _params?: {
        [k: string]: string | number | Function;
    };
};

export declare class XCone extends XMesh {
    constructor(data: IX3DObjectData, defaults: any);
    get _radius(): number | undefined;
    set _radius(val: number);
    get _height(): number | undefined;
    set _height(val: number);
}

export declare class XCylinder extends XMesh {
    constructor(data: IX3DObjectData, defaults: IX3DObjectData);
    set _radius_top(val: number);
    get _radius_top(): number | undefined;
    set _radius_bottom(val: number);
    get _radius_bottom(): number | undefined;
    set _height(val: number);
    get _height(): number | undefined;
}

/**
 * @property
 */
declare const XData: _XData;
export { XData }
export { XData as _xd }

export declare class _XData {
    #private;
    objects: XDataObject;
    variables: XDataVariable;
    constructor();
    /**
     * This method gets the XData object
     * @returns XDataObject object
     * @example
     *  // get the XDataObject object
     *  const o = XData._o["my-object-id"]
     *  // set the XDataObject object
     *  XData._o["my-object-id"] = {my:"object"}
     */
    get _o(): XDataObject;
    /**
     * This method adds an object to the XData object
     * @param objectId
     * @param object
     * @comment It is also possible to use the XData._o property -> XData._o["my-object-id"] = {my:"object"}
     * */
    set(objectId: string, object: any): void;
    /**
     * This method checks if the XData object has an object by id
     * @param objectId
     * @returns boolean
     * @comment It is also possible to query the XData._o property -> if(XData._o["my-object-id"])...
     * */
    has(objectId: string): boolean;
    /**
     * Deletes an object from the XData object
     * @param objectId
     */
    delete(objectId: string): void;
    /**
     * Gets an object and delete it from the XData object list
     * @param objectId
     * @returns
     */
    pick(objectId: string): any;
    /**
     * This method cleans the XData Memory
     */
    clean(): void;
}

/**
 * @class XDataBase - manage local storage
 */
export declare class _XDataBase {
    private _version;
    private _local;
    private _session;
    private _vectors;
    private _encode_context;
    constructor();
    info(): void;
    /**
     * Indicates if the data will be encoded before saving to the local storage
     */
    set encode(value: boolean);
    /**
     *
     * @param itemName
     * @param item
     * @deprecated use saveString instead
     */
    save(itemName: string, item: any): void;
    /**
     * loads value from local storage
     * @param itemName @
     * @returns string
     * @deprecated use getString() function instead
     */
    load(itemName: string): string | null;
    /**
     * Saves a string to the XDB
     * @param id The id of the string (tag)
     * @param stringValue the string value
     */
    saveString(id: string, stringValue: any): void;
    /**
     * Gets a string value from the data storage
     * @param id the id that the string was tagged with
     * @returns <string>
     */
    getString(id: string): string | null;
    /**
     * Gets a JSON Object
     * @param id the id of the object
     * @returns object (null if not exists)
     */
    getObject(id: string): any;
    /**
     * Saves an object to the XDB with an ID as a tag (will be used to retrieve the data back)
     * @param id id of the object (tag)
     * @param obj the object it self
     * @returns
     */
    saveObject(id: string, obj: object): boolean;
    resetAllData(): void;
}

declare type XDataInstanceXporter = {
    cls: any;
    handler: XDataXporterHandler;
};

/**
 * XData (Xpell Global shared Variables & Objects)
 * This object uses as a real-time shared memory between all Xpell modules nad components
 * Usage:
 *  - store primitive variable:
 *      XData._v["my-var-id"] = "my-var-value"
 *  - get primitive variable:
 *      const v = XData._v["my-var-id"]
 *  - store object:
 *      XData._o["my-object-id"] = {my:"object"}
 *  - get object:
 *      const o = XData._o["my-object-id"]
 */
export declare type XDataObject = {
    [_id: string]: any;
};

export declare type XDataVariable = {
    [_id: string]: string | number | boolean;
};

declare type XDataXporter = {
    _ignore_fields: string[];
    _instance_xporters: {
        [id: string]: XDataInstanceXporter;
    };
};

export declare interface XDataXporterHandler {
    (inst: any): any;
}

declare const XDB: _XDataBase;
export { XDB }
export { XDB as _xdb }

export declare class XEditor {
    static getObjects(): {
        "transform-controls": typeof XTransformControls;
    };
}

export declare type XEvent = {
    _id: number;
    _name: string;
    _data: any;
};

/**
 * This interface define the listener callable function (provided with "on" method)
 */
export declare interface XEventListener {
    _id?: string;
    _callback?: Function;
    _options?: XEventListenerOptions;
}

export declare type XEventListenerOptions = {
    _once?: boolean;
    _support_html?: boolean;
};

declare const XEventManager: _XEventManager;
export { XEventManager }
export { XEventManager as _xem }

/**
 * XEventDispatcher is the system event dispatcher and manager
 */
export declare class _XEventManager {
    _log_rules: {
        fire: any;
        register: boolean;
        remove: boolean;
    };
    protected _events: {
        [name: string]: Array<XEventListener>;
    };
    protected _listeners_to_event_index: {
        [listernerId: string]: string;
    };
    protected _html_event_listeners: HTMLEventListenersIndex;
    constructor();
    /**
     * This method listen to event name and register the listener function
     * @param eventName event name to listen to
     * @param listener listener function to be called when event fired
     * @returns listener id
     * @example
     *     // listen to event name "my-event" and display the event data to the console when fired
     *    _xem.on("my-event",(data)=>{
     *         console.log("XEM Event " + data)
     *    })
     */
    onEvent(eventName: string, listener: Function, options?: XEventListenerOptions, callObj?: any): string;
    /**
     * This method listen to event name and register the listener function
     * @param eventName event name to listen to
     * @param listener listener function to be called when event fired
     * @returns listener id
     * @example
     *     // listen to event name "my-event" and display the event data to the console when fired
     *    _xem.on("my-event",(data)=>{
     *         console.log("XEM Event " + data)
     *    })
     */
    on(eventName: string, listener: Function, options?: XEventListenerOptions, callObject?: any): string;
    /**
     * This method listen to event name and register the listener function
     * The listener will be removed after first fire
     * @param eventName event name to listen to
     * @param listener listener function to be called when event fired
     * @returns listener id
     */
    once(eventName: string, listener: Function, callObject?: any): string;
    /**
     * This method remove listener by listener id
     * @param listenerId listener id to remove
     */
    removeEvent(listenerId: string): void;
    /**
     *
     * @override
     * @param listenerId
     */
    remove(listenerId: string): void;
    /**
     *  This method is used to fire an event
     * @override
     * @param eventName - the name of the event
     * @param data - the data to pass to the event
     * @param supportHtmlEvents - if true the event will be fired as HTML event too
     */
    fire(eventName: string, data?: any, callObject?: any): Promise<void>;
}

export declare class XForm extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XGeometry extends X3DObject {
    width: number;
    height: number;
    depth: number;
    constructor(data: IX3DObjectData, defaults?: IX3DObjectData);
}

export declare class XGroup extends X3DObject {
    static getFromThreeObject(threeObject: THREE_2.Group, defaults: any): XGroup;
    constructor(data: IX3DObjectData, defaults?: {
        _type: string;
        _three_class: typeof THREE_2.Group;
        _three_obj: null;
    });
}

export declare class XHeader extends XUIObject {
    constructor(data: XObjectData);
}

/**
 * X3DHelper Data
 */
export declare type XHelperData<Type> = {
    _type: X3DHelpers;
    _active: boolean;
    _params?: {
        [Property in keyof Type]: Type[Property];
    };
};

export declare class XHTML extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XImage extends XUIObject {
    static xtype: string;
    constructor(data: XObjectData);
}

export declare class XInput extends XUIObject {
    constructor(data: XObjectData);
    set _text(text: string);
}

export declare class XLabel extends XUIObject {
    constructor(data: XObjectData);
}

/**
 * XLight - Xpell wrapper for Three Light
 */
export declare class XLight extends X3DObject {
    #private;
    static xtype: string;
    readonly _is_light: boolean;
    _three_obj: THREE_2.Light | null;
    _light?: XLightTypes;
    constructor(data: XLightData);
    set _color(color: number | string);
    get _color(): number | string;
    set _intensity(intensity: number);
    get _intensity(): number;
}

/**
 * XLight Data
 */
export declare interface XLightData extends IX3DObjectData {
    _light?: XLightTypes;
    _color?: number | string | THREE_2.Color;
    _intensity?: number;
}

/**
 * supported Three light type
 */
export declare type XLightTypes = "ambient" | "directional" | "spotlight" | "hemisphere" | "point" | "rectArea" | "lightE" | "lightProbe";

export declare class XLink extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XList extends XUIObject {
    _items: any;
    constructor(data: XObjectData);
}

/**
 *
 */
declare const XLogger: _XLogger;
export { XLogger }
export { XLogger as _xlog }

/**
 * Xpell-logger
 */
/**
 * @class XLoggerEngine Xpell Logger engine
 */
export declare class _XLogger {
    /**
     * Enable logger activity if false no logs will be displayed
     */
    enabled: boolean;
    /**
     * Show the date in every log message
     */
    showDate: boolean;
    /**
     * Show the Time in every log message
     */
    showTime: boolean;
    constructor();
    /**
     * Generates the log output date/time signature (affected by showDate & showTime properties)
     * @returns {string}
     */
    private getLogDateTimeSignature;
    /**
     * Log a message to the output log (console)
     * @param message - message to present
     * @param optionalParams
     */
    log(message?: any, ...optionalParams: any[]): void;
    /**
     * Log an error message to the output log (console)
     * @param message - message to present
     * @param optionalParams
     */
    error(message?: any, ...optionalParams: any[]): void;
}

/**
 * Material
 *
 * maps can be <string>  for file  or object with parameters to the texture
 * _texture_map : "/public/file-name"
 *
 * _texture_map: {
 *      texture:"/public/file-name",
 *      offset:$_v2 X Y //for Three.Vector2
 *      rotation: 3 //number
 * } */
export declare class XMaterial extends X3DObject {
    #private;
    _three_obj: any;
    constructor(data: IX3DObjectData);
    set _color(color: number | string);
}

export declare class XMesh extends X3DObject {
    static getFromThreeObject(threeObject: THREE_2.Mesh, defaults: any): XMesh;
    _geometry: XGeometry | IX3DObjectData;
    _material: XMaterial | IX3DObjectData;
    _wireframe: boolean;
    protected __color: number | undefined;
    constructor(data: IX3DObjectData, defaults?: {
        _type: string;
        _geometry: null;
        _material: null;
        _positional_audio_source: null;
    }, skipParse?: boolean);
    set _color(val: number);
    get _color(): number | undefined;
    getThreeObject(): Promise<THREE_2.Object3D<THREE_2.Event>>;
}

/**
 * XModel
 * enable auo wrapper for GLB/GLTF models
 * @example {
 *     _id: "model-id",
 *    _type: "xmodel",
 *   _model_url: "/public/models/scene.gltf",
 * }
 */
declare class XModel extends X3DObject {
    static xtype: string;
    constructor(data: IX3DObjectData);
}

/**
 * Xpell Base Module
 * This class represents xpell base module to be extends
 * @class XModule
 *
 */
export declare class XModule {
    #private;
    [k: string]: any;
    _id: string;
    _name: string;
    _log_rules: {
        createObject: boolean;
        removeObject: boolean;
    };
    constructor(data: XModuleData);
    load(): void;
    /**
     * Creates new XObject from data object
     * @param data - The data of the new object (JSON)
     * @return {XObject|*}
     */
    create(data: XObjectData): any;
    /**
     * removes and XObject from the object manager
     * @param objectId op
     */
    remove(objectId: string): void;
    _info(xCommand: XCommand): void;
    /**
     * Run xpell command -
     * CLI mode, parse the command to XCommand JSON format and call execute method
     * @param {string} XCommand input - text
     * @returns command execution result
     */
    run(stringXCommand: string): Promise<any>;
    /**
     * execute xpell command - CLI mode
     * @param {XCommand} XCommand input (JSON)
     * @returns command execution result
     */
    execute(xCommand: XCommand | XCommandData): Promise<any>;
    /**
     * This method triggers every frame from the Xpell engine.
     * The method can be override by the extending module to support extended onFrame functionality
     * @param frameNumber Current frame number
     */
    onFrame(frameNumber: number): Promise<void>;
    /**
     * X Object Manager
     */
    /**
     * getter for om (object manager) instance
     * @returns {XObjectManager}
     * @deprecated - use _object_manager instead
     * If you wish to get an object from the object manager use
     * getObject directly on the module instead of om.getObject
     */
    get om(): XObjectManager;
    get _object_manager(): XObjectManager;
    /**
     * Returns the XObject instance from the module Object Manager
     * @param objectId
     * @returns XObject
     */
    getObject(objectId: string): XObject;
    /**
     * Returns the XObject instance from the module Object Manager
     * Usage:
     * xmodule._o["object-id"] is equivalent to xmodule.getObject("object-id")
     */
    get _o(): XObjectManagerIndex;
    /**
     * Imports external object pack to the engine
     * The object class should be like XObjects with static implementation of getObjects() method
     * @param {XObjects} xObjectPack
     */
    importObjectPack(xObjectPack: XObjectPack | any): void;
    /**
     * Imports external object pack to the engine
     * @deprecated - use importObjectPack instead
     * @param xObjectPack
     */
    importObjects(xObjectPack: XObjectPack | any): void;
    /**
     * Imports external objects to the engine
     * The object class should be like XObjects with static implementation of getObjects() method
     * @param xObjectName
     * @param xObject
     */
    importObject(xObjectName: string, xObject: XObject): void;
}

export declare type XModuleData = {
    _name: string;
};

/**
 * Single x-nano-command interface
 */
export declare interface XNanoCommand {
    (xCommand: XCommand | XCommandData, xObject: XObject): any;
}

/**
 * x-nano-command pack
 */
export declare type XNanoCommandPack = {
    [k: string]: XNanoCommand;
};

export declare class XNavBar extends XUIObject {
    constructor(data: XObjectData);
}

/**
 * XObject class
 * @class XObject
 */
export declare class XObject {
    [k: string]: string | null | [] | undefined | Function | boolean | number | {} | null;
    _id: string;
    _type: string;
    _children: Array<XObject | XObjectData>;
    _parent: XObject | null;
    _name?: string;
    _data_source?: string;
    _on: XObjectOnEventIndex;
    _once: XObjectOnEventIndex;
    _on_create?: string | Function | undefined;
    _on_mount?: string | Function | undefined;
    _on_frame?: string | Function | undefined;
    _on_data?: string | Function | undefined;
    _on_event?: string | Function | undefined;
    _process_frame: boolean;
    _process_data: boolean;
    protected _xem_options: XEventListenerOptions;
    protected _nano_commands: {
        [k: string]: XNanoCommand;
    };
    protected _cache_cmd_txt?: string;
    protected _cache_jcmd?: any;
    protected _event_listeners_ids: {
        [eventName: string]: string;
    };
    protected _xporter: XDataXporter;
    /**
     * XObject constructor is creating the object and adding all the data keys to the XObject instance
     * @param data constructor input data (object)
     * @param defaults - defaults to merge with data
     * @param skipParse - skip data parsing
     * if override this method make sure to call super.init(data,skipParse) and to set skipParse to true
     */
    constructor(data: XObjectData, defaults?: any, skipParse?: boolean);
    /**
     * Initialize the XObject
     * @param data - data to parse (XObjectData)
     * @param skipParse - skip data parsing
     * @deprecated - use parse method instead
     */
    init(data?: any, skipParse?: boolean): void;
    parseEvents(options?: XEventListenerOptions): void;
    addEventListener(eventName: string, handler: XObjectOnEventHandler, options?: XEventListenerOptions): void;
    removeEventListener(eventName: string): void;
    removeAllEventListeners(): void;
    /**
     * Append a child XObject to this XObject
     * @param xobject
     */
    append(xobject: XObject): void;
    /**
     * Add single nano command to the object
     * @param commandName - the nano command name
     * @param nanoCommandFunction
     */
    addNanoCommand(commandName: string, nanoCommandFunction: XNanoCommand): void;
    addNanoCommandPack(ncPack: XNanoCommandPack): void;
    /**
     * List of fields to ignore when exporting the xobject to XData or string format
     * @param <string[]> ignoreFields - an array with all the fields to ignore
     */
    addXporterDataIgnoreFields(ignoreFields: string[]): void;
    /**
     * Add XData Xporter instance handler
     * @param <XDataInstanceXporter> ie - the instance exporter object
     */
    addXporterInstanceXporter(classOfInstance: any, handler: XDataXporterHandler): void;
    /**
     * Parse data to the XObject
     * @param data data to parse
     * @param ignore - lis of words to ignore in the parse process
     */
    parse(data: XObjectData, ignore?: any): void;
    /**
     * Parse data to the XObject
     * @param data data to parse
     * @param {object} fields- object with fields and default values (IXData format)
     *
     * fields example = {
     *  _name : "default-name",
     * ...
     * }
     */
    parseFieldsFromXDataObject(data: XObjectData, fields: {
        [name: string]: any;
    }): void;
    /**
     * Parse list of fields from IXObjectData to the class
     * @param {IXObjectData} data -  the data
     * @param {Array<string>} fields - array of field names (string)
     * @param checkNonXParams - also check non Xpell fields (fields that not starting with "_" sign)
     */
    parseFields(data: XObjectData, fields: Array<string>, checkNonXParams?: boolean): void;
    /**
     * this method triggered after the HTML DOM object has been created and added to the parent element
     * support external _on_create anonymous function in the , example:
     * _on_create: async (xObject) => {
     *      // xObject -> The XObject parent of the _on_create function, use instead of this keyword
     *      // write code that will be executed each frame.
     *      // make sure to write async anonymous function.
     * }
     *
     */
    onCreate(): Promise<void>;
    protected checkAndRunInternalFunction(func: any, ...params: any): Promise<void>;
    /**
     * Triggers when the object is being mounted to other element
     * support external _on_create anonymous function in the , example:
     * _on_mount: async (xObject) => {
     *      // xObject -> The XObject parent of the _on_mount function, use instead of this keyword
     *      // write code that will be executed each frame.
     *      // make sure to write async anonymous function.
     * }
     */
    onMount(): Promise<void>;
    emptyDataSource(): void;
    /**
     * Triggers when new data is being received from the data source
     * @param data - the data
     * if override this method make sure to call super.onData(data) to run the _on_data attribute
     */
    onData(data: any): Promise<void>;
    /**
     * Triggers from Xpell frame every frame
     * Support _on_frame atrribute that can be XCommand string or function
     * @param {number} frameNumber
     *
     * XObject supports
     * 1. External _on_frame anonymous function in the , example:
     * _on_frame: async (xObject,frameNumber) => {
     *      // xObject -> The XObject parent of the _on_frame function, use instead of this keyword
     *      // frameNumber = Xpell current frame number
     *      // write code that will be executed each frame.
     *      // make sure to write async anonymous function.
     *      // be wise with the function execution and try to keep it in the 15ms running time to support 60 FPS
     * }
     *
     * 2. String execution of nano commands
     *
     * _on_frame: "nano command text"
     *
     */
    onFrame(frameNumber: number): Promise<void>;
    /**
     * Runs object nano commands
     * @param nanoCommand - object nano command (string)
     * @param cache - cache last command to prevent multiple parsing on the same command
     */
    run(nanoCommand: string, cache?: boolean): Promise<void>;
    /**
     * Execute XCommand within the XObject Nano Commands
     * @param xCommand XCommand to execute
     *
     * Nano command example:
     *
     * "set-text" : (xCommand,xObject) => {
     *      xObject.setText(xCommands.params.text)
     * }
     *
     */
    execute(xCommand: XCommand | XCommandData): Promise<void>;
    /**
     * Return an IXObjectData JSON representation of the XObject
     * @returns IXObjectData
     */
    toXData(): IXData;
    /**
     * Return a string representation of the XObject
     * @returns string
     */
    toString(): string;
    clearAttributes(attributes: Array<string>): void;
    /**
     * Dispose the XObject and all its children
     */
    dispose(): Promise<void>;
}

export declare type XObjectData = {
    [k: string]: string | null | [] | undefined | Function | boolean | number | {};
    _id?: string;
    _type?: string;
    _children?: Array<XObject | XObjectData>;
    _name?: string;
    _data_source?: string;
    _on?: XObjectOnEventIndex;
    _once?: XObjectOnEventIndex;
    _on_create?: string | Function | undefined;
    _on_mount?: string | Function | undefined;
    _on_frame?: string | Function | undefined;
    _on_data?: string | Function | undefined;
    _process_frame?: boolean;
    _process_data?: boolean;
    _nano_commands?: XNanoCommandPack;
};

export declare class XObjectManager {
    #private;
    constructor();
    get _objects(): XObjectManagerIndex;
    /**
     * Checks if an object is found in the object manager
     * @param xObjectId
     * @returns
     */
    hasObject(xObjectId: string): boolean;
    /**
     * Register multiple classes dictionary into the object manager
     * @param xObjects - key value list -> \{"view":XView,...\}
     */
    registerObjects(xObjects: XObjectPack): void;
    /**
     * Registers single XObject
     * @param name - name of the object
     * @param xObjects The object class
     */
    registerObject(name: string, xObjects: XObject): void;
    /**
     * Checks if a class (name) is found in the object manager classes dictionary
     * @param name - class name
     * @returns {boolean}
     */
    hasObjectClass(name: string): boolean;
    /**
     * Retrieves XObject class instance
     * @param name class name
     * @returns {XObject}
     */
    getObjectClass(name: string): any;
    /**
     * Retrieves all the classes dictionary
     * @returns XObjectManagerIndex
     */
    getAllClasses(): XObjectManagerIndex;
    get _classes(): XObjectManagerIndex;
    /**
     * Add XObject instance to the manager
     * @param xObject XObject to maintain
     */
    addObject(xObject: XObject): void;
    /**
     * Remove XObject from the manager
     * @param xObjectId object id to remove
     */
    removeObject(xObjectId: string): void;
    /**
     * Retrieves XObject instance
     * @param xObjectId XObject id
     * @returns {XObject}
     */
    getObject(xObjectId: string): XObject;
    /**
     * alias to getObject
     * @param id
     * @returns
     */
    go(id: string): XObject;
    /**
     * Retrieves XObject instance
     * @param objectName XObject name
     * @returns {XObject}
     */
    getObjectByName(objectName: string): XObject | null;
}

/**
 * Xpell Module Object Manager
 * @description This manager holds the module XObjects that should be managed (XObject children will not be managed separately)
 * XModules uses the Object Manager to create new XObjects by providing the class of the object by name
 */
declare type XObjectManagerIndex = {
    [name: string]: any;
};

export declare type XObjectOnEventHandler = (xObject: XObject, data?: any) => void;

export declare interface XObjectOnEventIndex {
    [eventName: string]: XObjectOnEventHandler;
}

/**
 * ObjectPack class for multi object registration
 */
export declare class XObjectPack {
    [k: string]: any;
    /**
     * Get all registered object in this ObjectPack
     * @returns XObject dictionary
     */
    static getObjects(): object;
}

/**
 * Xpell Parser - Parse XML, HTML, Raw Text & Json to Xpell Command
 */
export declare class XParser {
    private static html2XMap;
    /**
     * Adds HTML-Xpell Mapping item
     * @param htmlElement HTML element to change from
     * @param xpellElement Xpell element to change to
     */
    static addHtml2XpellMapItem(htmlElement: string, xpellElement: string): void;
    /**
     * convert text command to Xpell json command
     * @param {string} txt
     */
    static parse(txt: string, module?: string): XCommand;
    static replaceSpacesInQuotes(inputString: string, replaceWith?: string): string;
    static parseXpellCommand(command: string, module?: string): {
        _module: string | undefined;
        _op: string | undefined;
        _params: any;
    };
    /**
     * Convert raw Xpell command (JSON) to XCommand
     * @param rawXpell JSON of Xpell command
     * @returns {XCommand}
     * @deprecated
     */
    /**
     * Parse CLI arguments
     * @param code arguments
     * @returns Array<string>
     * @deprecated
     */
    /**
     * Covent Xpell2 (XP2) Json to Xpell JSON
     * @param XP2Json Xpell 2 JSON
     * @returns
     * @deprecated
     */
    static xpellify(XP2Json: {
        [k: string]: any;
    }): any;
    /**
     * Converts XML/HTML string to XCommand
     * @param xmlString XML string
     * @returns
     */
    static xmlString2Xpell(xmlString: string): {};
    /**
     * Converts XML/HTML Document to Xpell JSON
     * @param xmlNode XML Document Node
     * @param forceXhtml force Xpell XHTML for every unknown object
     * @returns {} Xpell JSON
     */
    static xml2Xpell(xmlNode: any, forceXhtml?: boolean): {};
}

export declare class XPassword extends XUIObject {
    constructor(data: XObjectData);
    set _text(text: string);
}

/**
 * Xpell Engine instance
 * @public Xpell Engine instance
 */
declare const Xpell: XpellEngine;
export { Xpell }
export { Xpell as _x }
export default Xpell;

/**
 * @class  Xpell main engine
 */
declare class XpellEngine {
    #private;
    _version: string;
    _engine_id: string;
    _frame_number: number;
    _log_rules: {};
    parser: typeof XParser;
    constructor();
    /**
     * Enable Xpell logs to console
     */
    set verbose(val: boolean);
    /**
     * Logs message to console using Xpell logger
     * make sure to enable verbose mode to see the logs
     * this method is a wrapper for XLogger.log
     * @param msg
     * @param optionalParams
     */
    log(message?: any, ...optionalParams: any[]): void;
    /**
     * Delay the execution of the next command
     * @param ms - delay in milliseconds
     * @returns
     */
    delay(ms: number): Promise<unknown>;
    /**
     * Loads Xpell module into the engine
     * @param {XModule} xModule
     */
    loadModule(xModule: XModule): void;
    /**
     * Loads multiple module at ones
     * @param {Array<XModule>} xModulesArray
     */
    loadModules(...xModulesArray: Array<XModule>): void;
    /**
     * Display information about the Xpell engine to the console
     */
    info(): void;
    /**
     * Run textual xCommand -
     * @param {cmd} - text command
     */
    run(stringXCommand: string): any;
    /**
     * Execute Xpell Command
     * @param {XCommand}
     */
    execute(xcmd: XCommand): any;
    /**
     * Main onFrame method
     * calls all the sub-modules onFrame methods (if implemented)
     */
    onFrame(): void;
    /**
     * Gets Xpell module by name
     * @param {string} moduleName - name of the loaded module
     * @returns {XModule}
     */
    getModule(moduleName: string): XModule;
    /**
     * Start Xpell engine for web browsers using requestAnimationFrame
     */
    start(): void;
    getParam(name: string, defaultValue?: string): any;
}

export declare class XPlane extends XMesh {
    constructor(data: IX3DObjectData, defaults?: IX3DObjectData);
    set _width(val: number);
    get _width(): number | undefined;
    set _height(val: number);
    get _height(): number | undefined;
}

export declare class XSphere extends XMesh {
    private __radius?;
    constructor(data: IX3DObjectData, defaults: IX3DObjectData);
    set _radius(val: number);
    get _radius(): number | undefined;
}

export declare class XSVG extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGCircle extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGEllipse extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGLine extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGPath extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGPolygon extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGPolyline extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XSVGRect extends XUIObject {
    private _svg_data;
    constructor(data: XObjectData);
}

export declare class XTextArea extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XTextField extends XUIObject {
    constructor(data: XObjectData);
    set _text(text: string);
}

export declare class XTorus extends XMesh {
    constructor(data: IX3DObjectData, defaults: any);
    get _radius(): number | undefined;
    set _radius(val: number);
    get _tube_radius(): number | undefined;
    set _tube_radius(val: number);
}

export declare class XTransformControls extends XUIObject {
    private _keyboard_down_listener;
    private _key_down;
    private _state;
    constructor(data: IXObjectData);
    changeState(newState: 'translate' | 'rotate' | 'scale'): void;
    onMount(): Promise<void>;
}

/**
 * Xpell Web TTS (Text to Speech)
 */
export declare class XTTS extends XAIObject {
    static xtype: string;
    synth: SpeechSynthesis;
    voices: Array<SpeechSynthesisVoice>;
    constructor(data: XObjectData);
    onCreate(): Promise<void>;
    setPron(pron_dict: any): void;
    applyVoices(): void;
    loadVoices(): Promise<unknown>;
    setVoice(index: any): void;
    setVoiceByName(voiceName: string): void;
    speak(text: string, on_speech_word?: any, on_speech_end?: any): void;
    getVoices(): SpeechSynthesisVoice[];
}

export declare const XUI: XUIModule;

export declare const XUIAnimate: _AnimateCSS;

/**
 * XUIApp is the Xpell UI Application object, it can hold views and controls
 */
export declare type XUIApp = {
    xpell?: {
        version?: number;
    };
    _views?: XViewsPack;
    _controls?: {
        _parent_element: string | {};
        [k: string]: {} | string;
    };
};

/**
 * XUI Module - Xpell User Interface Module for HTML and CSS
 */
export declare class XUIModule extends XModule {
    vm: XViewManager;
    _first_gesture_occurred: boolean;
    _events: {
        _loaded: string;
    };
    private _controls_element;
    private _player_element;
    /**
     * @fires "xui-loaded" event
     * @param data module data
     */
    constructor(data: XModuleData);
    /**
     * Create a XUIObject and mount it to the DOM parent element.
     * if xData._parent_element is not provided the object will be appended to the player element or to the document body if player element is not provided
     * @param data - the XUIObject data
     * @returns XUIObject
     *
     */
    add(xData: XObjectData): any;
    /**
     * Append XUIObject to the parent XUI Object
     * @param xobj - XUIObject to append
     * @param parentXobjId - the parent XUIObject id
     */
    append(xobj: XUIObject | XObjectData, parentXobjId: string): void;
    /**
     * Create a XUIObject
     * @param data - XOjectData representing the XUIObject
     * @returns XUIObject
     */
    create(data?: XObjectData): any;
    /**
     * Wraps an array of XObjectData objects with a wrapper object and returns the wrapper
     * with the wrapped objects as children
     * @param xObjects - array of XObjectData objects to wrap
     * @param wrapper - the wrapper object to use, if not provided a default wrapper will be created
     * @returns - XObjectData
     */
    wrap(xObjects: (XObjectData)[], wrapper?: XObjectData): XObjectData;
    /**
     * Loads Xpell application object
     * @param xuiApp
     */
    loadApp(xuiApp: XUIApp): void;
    /**
     *
     * @param url navigate the browser to new url
     * @param newWindow - if provided and true the url will be opened in a new window/tab
     */
    openUrl(url: string, newWindow?: boolean): void;
    /**
     * Removes the XUIObject from the DOM by ID and from the Object Manager
     * @param objectId the XUIObject id to remove
     * @override
     */
    remove(objectId: string): void;
    addControlsPack(controls: {
        [name: string]: {};
    }): void;
    /**
     * Create a XUIObject and mount it to the DOM parent element
     * @param data - the XUIObject data
     * @returns XUIObject
     * @deprecated use XUI.add instead
     */
    loadControl(data: XObjectData): XUIObject;
    /**
     * Create a XUIObject and mount it to the DOM parent element
     * @param data - the XUIObject data
     * @returns XUIObject
     * @deprecated use XUI.add instead
     */
    loadObject(data: XObjectData): XUIObject;
    createFromTemplate(xpell2json: {
        [k: string]: any;
    }): any;
    /**
     * The method fires "first-user-gesture" event
     * This method is for all Web API that requires User Gesture event.
     * @fire "first-user-gesture"
     */
    enableFirstUserGestureEvent(): void;
    /**
     * This method creates a player element and append it to the DOM
     * @param playerId - optional id of the player element
     * @param parentElementId - optional id of the element to append the player to, if not provided the player will be appended to the body
     * @returns HTMLDivElement
     */
    createPlayer(playerId?: string, cssClass?: string, parentElementId?: string): HTMLDivElement;
    /**
     * Show's XUIObject that was loaded into the XUI Object Manager
     * @param objectId - the XUIObject id to show
     */
    show(objectId: string): void;
    /**
     * Hide's XUIObject that was loaded into the XUI Object Manager
     * @param objectId - the XUIObject id to hide
     */
    hide(objectId: string): void;
    /**
     * Toggle's XUIObject that was loaded into the XUI Object Manager
     * @param objectId - the XUIObject id to toggle
     */
    toggle(objectId: string): void;
}

/**
 *   ADD On Event support
 *  - override addEventListener to add html event listener if object exist
 *  - override removeEventListener to remove html event listener
 *  - check for events in getDOMObject and add them to the object
 */
export declare class XUIObject extends XObject {
    #private;
    _html_tag: string;
    _html_ns?: string | null;
    protected _dom_object: any;
    _type: string;
    _html?: string | undefined;
    _base_display?: string | undefined | null;
    _children: XUIObject[];
    _visible: boolean;
    _parent_element?: string;
    _on_click?: Function | string;
    _on_show?: Function | string;
    _on_hide?: Function | string;
    _on_show_animation?: string;
    _on_hide_animation?: string;
    constructor(data: XObjectData, defaults: XObjectData, skipParse?: boolean);
    /**
     * Dispose all object memory (destructor)
     */
    dispose(): Promise<void>;
    /**
     * logs the object to the console
     */
    log(): void;
    /**
     * Gets the HTML DOM object, if the object is not created yet it will be created
     * @returns the HTML DOM object
     */
    getDOMObject(): HTMLElement;
    /**
     * DOM Getter
     * @returns the HTML DOM object same as getDOMObject()
     */
    get dom(): HTMLElement;
    set _text(text: string);
    get _text(): string;
    /**
     * Gets the HTML representation of the object
     * @returns the HTML representation of the object
     */
    getHTML(): string;
    /**
     * Attach the object to HTML element
     * @param parentElementId
     * @deprecated use "mount" function instead
     */
    attach(parentElementId: string): void;
    /**
     * Mount the object to HTML element
     * @param parentElementId
     *
     */
    mount(parentElementId: string): void;
    /**
     * Append a child object to the XUIObject, if the object is not XUIObject it will be created
     * @param xObject - the child object to append can be XUIObject or XObjectData
     * @returns
     */
    append(xObject: XUIObject | XObjectData | any): any;
    /**
     * Sets the object text content
     * @param text - the text content
     * @deprecated use _text property instead (e.g. xuiObj._text = "Xpell rulz!")
     */
    setText(text: string): void;
    /**
     * Sets the object CSS style
     * @param attr - the CSS attribute
     * @param val - the CSS value
     * @example
     * xuiObj.setStyle("background-color","red")
     */
    setStyleAttribute(attr: string, val: string): void;
    /**
     * Adds a css class to the object
     * @param className - the css class name
     */
    addClass(className: string): void;
    /**
     * Removes a css class from the object
     * @param className - the css class name
     */
    removeClass(className: string): void;
    /**
     * Toggles a css class on the object
     * @param className - the css class name
     */
    toggleClass(className: string): void;
    /**
     * Replaces a css class on the object
     * @param oldClass class to be replaced
     * @param newClass new class to replace the old class
     */
    replaceClass(oldClass: string, newClass: string): void;
    /**
     * This method is used to show the object and trigger the onShow event
     */
    show(): void;
    /**
     * This method is used to hide the object and trigger the onHide event
     */
    hide(): void;
    animate(animation: string, infinite?: boolean): Promise<unknown>;
    stopAnimation(): void;
    /**
     * This method is used to toggle the object visibility
     */
    toggle(): void;
    click(): void;
    /**
     * this method triggered after the HTML DOM object has been mounted by the super
     * it implemented in this class to support the following events for XUIObject:
     * _on_click: (XUIObject,event) => {}
     */
    onMount(): Promise<void>;
    /**
     * this method triggered when the XUIObject is shown
     */
    onShow(): Promise<void>;
    /**
     * this method triggered when the XUIObject is hidden
     */
    onHide(): Promise<void>;
}

export declare class XUIObjects extends XObjectPack {
    static getObjects(): {
        view: typeof XView;
        div: typeof XView;
        label: typeof XLabel;
        link: typeof XLink;
        button: typeof XButton;
        text: typeof XTextField;
        password: typeof XPassword;
        input: typeof XInput;
        textarea: typeof XTextArea;
        video: typeof XVideo;
        image: typeof XImage;
        list: typeof XList;
        form: typeof XForm;
        webcam: typeof XWebcam;
        xhtml: typeof XHTML;
        svg: typeof XSVG;
        circle: typeof XSVGCircle;
        rect: typeof XSVGRect;
        ellipse: typeof XSVGEllipse;
        line: typeof XSVGLine;
        polyline: typeof XSVGPolyline;
        polygon: typeof XSVGPolygon;
        path: typeof XSVGPath;
    };
}

declare class XUtils {
    /**
     * create ignore list for parser to ignore spells words
     * @param list - list of reserved words (comma separated)
     */
    static createIgnoreList(list: string, reservedWords: {}): {
        [k: string]: string;
    };
    /**
     * Generates GUID (Globally unique Identifier)
     * @returns {string}
     */
    static guid(): string;
    /**
     * Merges XDataObject with Defaults object
     * @param data - data of the Xpell command
     * @param defaults - defaults object to merge with
     * @param force - add defaults values even if exists
     */
    static mergeDefaultsWithData(data: IXData_2, defaults: IXData_2, force?: boolean): void;
    /**
     * Encode string to Base-64 format
     * @param str string to encode
     * @returns {string}
     */
    static encode(str: string): string;
    /**
     * Decode Base64 String to text
     * @param str Base64 encoded string
     * @returns {string}
     */
    static decode(str: string): string;
    /**
     * Returns a random integer between min (inclusive) and max (inclusive).
     * The value is no lower than min (or the next integer greater than min
     * if min isn't an integer) and no greater than max (or the next integer
     * lower than max if max isn't an integer).
     * Using Math.round() will give you a non-uniform distribution!
     */
    static getRandomInt(min: number, max: number): number;
    /**
     * Extracts parameter from XCommand
     * @param xcmd - XCommand object
     * @param paramName - The name of the parameter to extract
     * @param defaultValue - Default value if parameter is not found
     * @returns
     */
    static getParam(xcmd: XCommand, paramName: string, defaultValue?: any): any;
}
export { XUtils }
export { XUtils as _xu }

export declare class XVector3 {
    _x: number;
    _y: number;
    _z: number;
    constructor(data?: XVector3Data);
    fromThreeVector3(tv3: THREE_2.Vector3): void;
    fromThreeEuler(euler: THREE_2.Euler): void;
    fromXData(data: XVector3Data): void;
    toThreeVector3(): THREE_2.Vector3;
    toThreeEuler(): THREE_2.Euler;
    toXData(): XVector3Data;
    toString(): string;
}

export declare type XVector3Data = {
    x: number;
    y: number;
    z: number;
};

export declare class XVideo extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XView extends XUIObject {
    constructor(data: XObjectData);
}

export declare class XViewManager {
    #private;
    _log_rules: {
        _init: boolean;
        _create: boolean;
    };
    constructor();
    get _id(): string;
    get _active_view(): string | null;
    get _default_parent_element(): string;
    /**
     * Initialized the View Manager and register "hashchange" event on the window to control the url string
     * @fire "xui-vm-loaded" event
     */
    init(): void;
    /**
     * Creates new SpellView
     * @description turns view-data (JSON) to a spell object
     * @param viewData
     * @param auto_add - if true and the view data (view_data) contains a "name" string the new view will be added automatically to the view manager
     * @return {XView}
     */
    createView(viewData: XObjectData, auto_add?: boolean): Promise<XUIObject>;
    /**
     * Adds an instance of a view (XView) to the View Manager
     * @param {XView}view The view instance
     * @param {string}viewName The view name
     */
    addView(view: XUIObjects, viewName: string): void;
    /**
     * Retrieve the view instance from the View Manager
     * @param viewName The view name to retrieve
     * @returns XView
     */
    getView(viewName: string): XUIObjects;
    /**
     * Checks if there is an instance of a view in the View Manager
     * @param viewName The view name
     * @returns
     */
    hasView(viewName: string): boolean;
    addViewPack(vuz: XViewsPack): void;
    addRawView(viewName: string, viewData: XObjectData): void;
    /**
     * Load page (entire screen) on top of the active page
     * This method handles the first routing where the view name is in the url (http://server:port/#view-name)
     * @param defaultViewName this param is the default view to load in case the url param is empty
     */
    loadPage(defaultViewName: string): void;
    /**
     * handle the hashchange browser event, used to support Back functionality.
     */
    onBrowserUrlHashChanged(): void;
    /**
     * Show view on screen
     * @param {*} viewName
     */
    showView(viewName: string): Promise<void>;
    /**
     * Show view on screen
     * @param {*} viewName
     */
    hideView(viewName: string): void;
    /**
     * Hide the active page and the page URL
     * @param {*} viewName
     */
    hidePage(viewName: string): void;
    /**
     * Show view as page (set as active view and dismiss former active)
     * @param {*} viewName
     */
    showPage(viewName: string): Promise<XUIObjects>;
}

export declare type XViewsPack = {
    _parent_element: string | {};
    [k: string]: {} | string;
};

export declare class XWebcam extends XUIObject {
    autoplay: boolean;
    muted: boolean;
    _video_constraints: {
        video: boolean;
        width: number;
        height: number;
    };
    constructor(data: XObjectData);
    /**
     * this method checks if the browser has "getMedia" support for webcam & mic
     * @returns boolean
     */
    isAvailable(): boolean;
    setSource(video_elem: any): void;
    get isPlaying(): boolean;
    pause(): Promise<void>;
}

/**
 * Spell3dWorld
 */
export declare const XWorldStatus: {
    New: number;
    Running: number;
    Paused: number;
};

export { }
