<?php

/**
 * XCommand class - this command is being sent to the Xpell parser or every XModule/XObject for execution
 */
class XCommand {

    /**
     * The XModule to handle the command
     * @var string
     */
    public string $_module;

    /**
     * The XObject that should handle the command (optional - uses only to send XCommand to specific object)
     * @var string|null
     */
    public ?string $_object = null;

    /**
     * The command operation (op/method) to execute
     * @var string
     */
    public string $_op;

    /**
     * Command parameters array
     * @var array|null
     */
    public ?array $_params = null;

    /**
     * XCommand create date timestamp
     * @var int
     */
    public int $d;

    /**
     * Constructor for XCommand
     * @param array|null $data
     */
    public function __construct(array $data = null) {
        if ($data) {
            foreach ($data as $key => $value) {
                if (property_exists($this, $key)) {
                    $this->$key = $value;
                }
            }
        }
        if (!isset($this->d)) {
            $this->d = time() * 1000; // Timestamp in milliseconds
        }
    }

    /**
     * Gets the parameter value from the XCommand whether it has a name or just a position
     * @param int $position The position of the parameter if no name is sent
     * @param string $name The name of the parameter
     * @param mixed $defaultValue The default value if none above exists
     * @return mixed The actual parameter value
     */
    public function getParam(int $position, string $name, $defaultValue) {
        if (isset($this->_params[$name])) {
            return $this->_params[$name];
        } elseif (isset($this->_params[$position])) {
            return $this->_params[$position];
        } else {
            return $defaultValue;
        }
    }
}
