<?php
/**
 * XData (Xpell Global shared Variables & Objects)
 * This object uses as a real-time shared memory between all Xpell modules and components
 * Usage:
 *  - store primitive variable: 
 *      XData::$variables["my-var-id"] = "my-var-value"
 *  - get primitive variable:
 *      $v = XData::$variables["my-var-id"]
 *  - store object:
 *      XData::$objects["my-object-id"] = ["my" => "object"]
 *  - get object:
 *      $o = XData::$objects["my-object-id"]
 */

class XData {
    // Deprecated, use $objects instead
    public static $variables = [];
    public static $objects = [];

    private static $_objects = [];

    /**
     * This method gets the XData object
     * @return array
     * @example
     *  // get the XDataObject object
     *  $o = XData::$objects["my-object-id"];
     *  // set the XDataObject object
     *  XData::$objects["my-object-id"] = ["my" => "object"];
     */
    public static function getObjects() {
        return self::$_objects;
    }


    /**
     * This method gets an object by id
     * @param string $objectId
     * @return mixed|null
     */
    public static function _o($objectId) {
        return self::$_objects[$objectId] ?? null;
    }


    /**
     * This method adds an object to the XData object
     * @param string $objectId
     * @param mixed $object
     * @comment It is also possible to use the XData::$objects property -> XData::$objects["my-object-id"] = ["my" => "object"]
     */
    public static function set($objectId, $object) {
        self::$_objects[$objectId] = $object;
    }

    /**
     * This method checks if the XData object has an object by id
     * @param string $objectId
     * @return bool
     * @comment It is also possible to query the XData::$objects property -> if(isset(XData::$objects["my-object-id"]))...
     */
    public static function has($objectId) {
        return array_key_exists($objectId, self::$_objects);
    }

    /**
     * Deletes an object from the XData object
     * @param string $objectId
     */
    public static function delete($objectId) {
        unset(self::$_objects[$objectId]);
    }

    /**
     * Gets an object and delete it from the XData object list
     * @param string $objectId
     * @return mixed
     */
    public static function pick($objectId) {
        $obj = self::$_objects[$objectId];
        self::delete($objectId);
        return $obj;
    }

    /**
     * This method cleans the XData Memory
     */
    public static function clean() {
        self::$_objects = [];
    }
}

// Initialize the XData object
XData::clean();
?>