<?php
/**
 * Xpell-logger
 */

/**
 * @class XLoggerEngine Xpell Logger engine
 */
class XLogger {
    /**
     * Enable logger activity if false no logs will be displayed
     */
    public static $enabled = true;

    /**
     * Show the date in every log message
     */
    public static $showDate = false;

    /**
     * Show the Time in every log message
     */
    public static $showTime = true;

    /**
     * Generates the log output date/time signature (affected by showDate & showTime properties)
     * @returns string
     */
    private static function getLogDateTimeSignature() {
        $d = new DateTime();

        $getDate = function() use ($d) {
            return (self::$showDate) ? $d->format('d.m.Y ') : "";
        };

        $getTime = function() use ($d) {
            return (self::$showTime) ? $d->format('H:i:s.v|') : "";
        };

        return $getDate() . $getTime();
    }

    /**
     * Log a message to the output log (console)
     * @param mixed $message - message to present
     * @param mixed ...$optionalParams
     */
    public static function log($message = null, ...$optionalParams) {
        if (self::$enabled) {
            $args = func_get_args();
            array_unshift($args, self::getLogDateTimeSignature());
            error_log(implode(' ', $args));
        }
    }

    /**
     * Log an error message to the output log (console)
     * @param mixed $message - message to present
     * @param mixed ...$optionalParams
     */
    public static function error($message = null, ...$optionalParams) {
        $args = func_get_args();
        array_unshift($args, self::getLogDateTimeSignature());
        error_log(implode(' ', $args));
    }
}

?>