<?
// @codingStandardsIgnoreStart
require_once (plugin_dir_path(__FILE__) .'XUIObject.php');
require_once (plugin_dir_path(__FILE__) .'XUICoreObjects.php');
require_once (plugin_dir_path(__FILE__) .'../XModule.php');
require_once (plugin_dir_path(__FILE__) .'../XEventManager.php');
require_once (plugin_dir_path(__FILE__) .'../XLogger.php');
// @codingStandardsIgnoreEnd



use XEventManager as _xem;
use XLogger as _xlog;


class XUI extends XModule {
    
    private static $_instance;
    
    public static function get_instance() {
        if (!self::$_instance) {
            self::$_instance = new self(['_name' => 'XUI']);
        }
        return self::$_instance;
    }


    private $_events = [
        '_loaded' => 'xui-loaded',
    ];
    private $_controls_element;
    private $_player_element;

    
    public function __construct($data) {
        parent::__construct($data);
        $this->importObjectPack(new XUICoreObjects());
        _xem::fire($this->_events['_loaded']);
    }

    public function add($xData) {
        
        $xobj = $this->create($xData);
        
        if (method_exists($xobj, 'onMount')) {
            $xobj->onMount();
        }
        $ctrl = $xobj->getDOMObject();
        
        return $ctrl;
    }

    public function append($xobj, $parentXobjId) {
        $parent = $this->getObject($parentXobjId);
        if ($parent) {
            $parent->append($xobj);
        } else {
            _xlog::log("XUI.append| Parent object " . $parentXobjId . " not found for object " . $xobj->_id);
        }
    }

    public function create($data = null) {
        if (!$data) {
            $data = ['_type' => 'view', '_children' => []];
        } else if (!array_key_exists('_type', $data)) {
            $data['_type'] = 'view';
        }
        return parent::create($data);
    }

    public function wrap($xObjects, $wrapper = null) {
        if (!$wrapper) {
            $wrapper = ['_type' => 'view', 'class' => 'xflex', '_children' => []];
        } else {
            if (!array_key_exists('_children', $wrapper)) $wrapper['_children'] = [];
            if (!array_key_exists('class', $wrapper)) $wrapper['class'] = 'xflex';
            else $wrapper['class'] .= ' xflex';
            if (!array_key_exists('_type', $wrapper)) $wrapper['_type'] = 'view';
        }
        foreach ($xObjects as $xobj) {
            $wrapper['_children'][] = $xobj;
        }
        return $wrapper;
    }

    public function loadApp($xuiApp) {
        // if (array_key_exists('_views', $xuiApp)) {
        //     $this->vm->addViewPack($xuiApp['_views']);
        // }
        // if (array_key_exists('_controls', $xuiApp)) {
        //     $this->addControlsPack($xuiApp['_controls']);
        // }
        _xem::fire('xui-app-loaded');
    }

    

    public function remove($objectId) {
        if ($this->_log_rules['removeObject']) {
            _xlog::log("XUI remove object " . $objectId);
        }
        parent::remove($objectId);
    }

    

    public function createPlayer($playerId = "xplayer", $cssClass = null, $parentElementId = null) {
        $div = "<div id='" . $playerId . "' class='" . (($cssClass) ? $cssClass : $playerId) . "'></div>";
        return $div;
    }
}


?>