<?php
/**
 * XUIObject - XUI Object is the base class for all XUI objects
 * @class XUIObject
 * @extends XObject
 * @autor Tamir Fridman
 * @since  22/07/2022
 * @copyright Aime Technologies 2022, all right reserved
 */
require_once (plugin_dir_path(__FILE__) .'../XObject.php');
require_once (plugin_dir_path(__FILE__) .'../XEventManager.php');



class XUIObject extends XObject {

    public $_id;
    public $_type;
    public $_children = [];
    public $_parent = null;
    public $_name;
    public $_data_source;
    public $_on;
    public $_once;
    public $_on_create;
    public $_on_mount;
    public $_on_frame;
    public $_on_data;
    public $_on_event;
    public $_process_frame;
    public $_process_data;
    public $style;
    public $class;
    protected $_xem_options;
    protected $_nano_commands = [];
    protected $_cache_cmd_txt;
    protected $_cache_jcmd;
    protected $_event_listeners_ids = [];
    protected $_xporter;

    public $_html_tag = "div";
    public $_html_ns = null;
    protected $_dom_object;
    public $_html;
    public $_base_display;
    public $_text = "";
    
    private $_initial_data;


    private $reservedWords = [ '_children' => "child objects" ];
    private $xpellObjectHtmlFieldsMapping = [ "_id" => "id", "css-class" => "class", "animation" => "xyz", "input-type" => "type" ];
    

    public function __construct($data, $defaults, $skipParse = false) {        
        parent::__construct($data, $defaults, true);
        // $this->addXporterDataIgnoreFields(["_dom_object", "_html", "_xem_options", "_on_click", "#_text"]);
        $this->_initial_data = $data;
        if (!$skipParse && $data) {
            $this->parse($data, ['_children' => 'child objects']);
        }
    }

    public function dispose() {
        $this->_dom_object = null;
        $this->_html = null;
        $this->_base_display = null;
        $this->_text = "";
        $this->_on_click = null;
        $this->_on_show = null;
        $this->_on_hide = null;

        parent::dispose();
    }


    public function getDOMObject() {
        if (!$this->_dom_object) {
            $dom_object = "<" . $this->_html_tag . " ";
            $fields = array_keys(get_object_vars($this));

            foreach ($fields as $field) {
                //check if the field is in the class and in the initial data
                if ($this->$field && isset($this->_initial_data[$field])) {
                    $f_out = $field;
                    if (isset($this->xpellObjectHtmlFieldsMapping[$field])) {
                        $f_out = $this->xpellObjectHtmlFieldsMapping[$field];
                    }
                    if (!str_starts_with($f_out, "_")) {
                        //replace quotes with /" to avoid html parsing issues
                        $dom_object .= " " .  strval($f_out) . '="' . str_replace('"', '\\"', $this->$field) . '"';
                    }
                }
            }

            $dom_object .= ">";
            if ($this->_text && strlen($this->_text) > 0) {
                //escape tet to avoid html parsing issues
                $dom_object .= str_replace("<", "&lt;", str_replace(">", "&gt;", $this->_text));
            }

            if (count($this->_children) > 0) {
                foreach ($this->_children as $child) {
                    $coo = $child->getDOMObject();
                    $dom_object .= $coo;
                }
            }
            $dom_object .= "</" . $this->_html_tag . ">";
            $this->_dom_object = $dom_object;
        }
        return $this->_dom_object;
    }

    public function  dom() {
        return $this->getDOMObject();
    }

    

    public function getHTML() {
        $dom = $this->getDOMObject();
        $this->_html = $dom->outerHTML;
        return $this->_html;
    }


    public function append($xObject) {
       
        parent::append($xObject);
        if ($this->_dom_object instanceof \DOMElement) {
            $this->_dom_object->appendChild($xObject->dom);
            $xObject->onMount();
        } else {
            return $xObject;
        }
    }

    public function removeChild($xObject) {
        
        parent::remove($xObject);
    }

    public function setText($text) {
        $this->_text = $text;
    }

    public function setStyleAttribute($attr, $val) {
        if (!$this->style) {
            $this->style = "";
        }
        $this->style .= $attr . ":" . $val . ";";
    }

    public function addClass($className) {
        if (!$this->class) {
            $this->class = $className;
        } else if (strpos($this->class, $className) === false) {
            $this->class .= " " . $className;
        }
    }

    public function removeClass($className) {
        if($this->class) {
            $this->class = str_replace($className, "", $this->class);
        }
    }

    public function replaceClass($oldClass, $newClass) {
        if($this->class) {
            $this->class = str_replace($oldClass, $newClass, $this->class);
        }
    }
}
?>