<?php
/**
 * Xpell - Real-Time User Interface Platform
 * PHP Edition
 * 
 * @file xpell.php
 * @overview Xpell Engine main file
 * @description Xpell Engine is a real-time user interface platform 
 * for PHP supporting devices & browsers.
 * @since  22/07/2022
 * @author Fridman Tamir
 * @license GNU GPL v3
 */

 
require_once (plugin_dir_path(__FILE__) .'XCommand.php');
require_once (plugin_dir_path(__FILE__) .'XUtils.php');
require_once (plugin_dir_path(__FILE__) .'XLogger.php');
require_once (plugin_dir_path(__FILE__) .'XData.php');
require_once (plugin_dir_path(__FILE__) .'XParser.php');
require_once (plugin_dir_path(__FILE__) .'XModule.php');
require_once (plugin_dir_path(__FILE__) .'XEventManager.php');
require_once (plugin_dir_path(__FILE__) .'XUI/XUI.php');

class Xpell {
    private static $_version;
    private static $_engine_id;
    private static $_frame_number;
    private static $_log_rules = array();
    
    private static $_modules = array();
    public static $parser;

    public static function init() {
        global $XUI;
        self::$_version = "0.0.1";
        self::$_engine_id = XUtils::guid();
        self::$_frame_number = 0;

        self::$parser = new XParser();
        self::$_modules = array();
        XEventManager::fire("xpell-init");
        XLogger::$enabled = true;
        self::loadModule(XUI::get_instance());
    }

    public static function setVerbose($val) {
        XLogger::$enabled = true;
    }

    public static function log($message, ...$optionalParams) {
        XLogger::log($message, ...$optionalParams);
    }

    public static function delay($ms) {
        usleep($ms * 1000);
    }

    public static function loadModule($xModule) {
        if (array_key_exists($xModule->_name, self::$_modules)) {
            XLogger::log("Module " . $xModule->_name . " already loaded");
        } else {
            self::$_modules[$xModule->_name] = $xModule;
            $xModule->load();
        }
    }

    public static function loadModules(...$xModulesArray) {
        foreach ($xModulesArray as $mod) {
            self::loadModule($mod);
        }
    }

    public static function info() {
        XLogger::log("Xpell information:\n- Engine Id: " . self::$_engine_id . "\n- Version " . self::$_version);
    }

    public static function run($stringXCommand) {
        if (strlen($stringXCommand) > 2) {
            $scmd = XParser::parse($stringXCommand);
            return self::execute($scmd);
        } else {
            throw new Exception("Unable to parse Xpell command");
        }
    }

    public static function execute($xcmd) {
        if ($xcmd && $xcmd->_module && array_key_exists($xcmd->_module, self::$_modules)) {
            return self::$_modules[$xcmd->_module]->execute($xcmd);
        } else {
            throw new Exception("Xpell module " . $xcmd->_module . " not loaded");
        }
    }

    public static function onFrame() {
        self::$_frame_number++;
        foreach (self::$_modules as $mod) {
            if (method_exists($mod, 'onFrame')) {
                $mod->onFrame(self::$_frame_number);
            }
        }
        XData::$objects["frame-number"] = self::$_frame_number;
        // Simulate requestAnimationFrame
        usleep(1000 / 60 * 1000); // 60 FPS
        self::onFrame();
    }

    public static function getModule($moduleName) {
        return self::$_modules[$moduleName];
    }

    public static function start() {
        XLogger::log("Starting Xpell");
        self::onFrame();
    }

    public static function getParam($name, $defaultValue = null) {
        return XData::$_o[$name] ?? $defaultValue;
    }
}



?>