<?php

require_once(plugin_dir_path(__FILE__) . 'xwp.php');

class AimeMenu
{

    // Properties

    //constructor
    public function __construct()
    {
        add_action('admin_post_save_aime_settings', array($this, 'handle_aime_settings_form'));
    }

    public function register_menu()
    {
        add_action('admin_menu', array($this, 'add_admin_menu'));
    }


    public function add_admin_menu()
    {
        /**
         * To make sure that the icon gets the Wordpress admin color scheme (color)
         * it is required to use the svg file as a base64 encoded string.
         * The Icon should also be 20x20 pixels and have a transparent background (svg fill="currentColor").
         * - read the svg file and encode it to base64 
         * */
        if ( ! function_exists( 'WP_Filesystem' ) ) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }
        
        global $wp_filesystem;
        WP_Filesystem();
        
        $icon_file = plugin_dir_path(__FILE__) . 'images/xpell-logo-wp-admin-menu.svg';
        
        // Check if file exists and use WP_Filesystem to read it
        if ( $wp_filesystem->exists( $icon_file ) ) {
            $icon_data = $wp_filesystem->get_contents( $icon_file );
            $icon_url = 'data:image/svg+xml;base64,' . base64_encode( $icon_data );
        } else {
            $icon_url = ''; // Handle missing file case
        }

        add_menu_page(
            'Xpell.AI', //page title
            'Xpell.AI',
            'add_users', //capability 
            'xpell-ai', //slug
            array($this, 'aime_menu_page'),  //callback
            $icon_url, //icon
            6 //position
        );

        add_submenu_page(
            'xpell-ai', /* Parent slug */
            esc_html__('Settings', 'xpell-ai'), /* Page title */
            esc_html__('Settings', 'xpell-ai'), /* Menu title */
            'manage_options', /* Capability */
            'xpell-ai-settings', /* Unique Menu slug */
            array($this, 'aime_settings_menu') /* Callback */
        );
        add_submenu_page(
            'xpell-ai', /* Parent slug */
            esc_html__('Connect', 'xpell-ai'), /* Page title */
            esc_html__('Connect', 'xpell-ai'), /* Menu title */
            'manage_options', /* Capability */
            'xpell-ai-connect', /* Unique Menu slug */
            array($this, 'aime_connect_menu') /* Callback */
        );
        add_submenu_page(
            'xpell-ai', /* Parent slug */
            esc_html__('Context', 'xpell-ai'), /* Page title */
            esc_html__('Context', 'xpell-ai'), /* Menu title */
            'manage_options', /* Capability */
            'xpell-menu-context', /* Unique Menu slug */
            array($this, 'aime_context_menu') /* Callback */
        );
    }

    function aime_menu_page()
    {

        $aimeSettings = get_option('aime_settings');

?>


        <div id="floating-div" class="floating-box">
            <h1> Xpell.AI Dashboard </h1>
        </div>

    <?php
    }

    function aime_settings_menu()
    {
        global $_xwp;
        $scripts = $_xwp->get_aime_scripts_urls();

        $aime_settings = get_option('aime_settings');
        if (!$aime_settings) {
            $aime_settings = array();
            $aime_settings['api_key'] = false;
            $aime_settings['owner_id'] = '';
            $aime_settings['aime_server_url'] = 'https://mat.aimeverse.com';
            $aime_settings['first_name'] = '';
            $aime_settings['last_name'] = '';
        }



    ?>
        <script type="module" defer>
            import {_xd,_x,_xlog} from "<?php echo esc_html($scripts['xpell_script']) ?>";


            _xd._o["aime_public_folder"] = "<?php echo esc_html($scripts['aime_public_folder']) ?>";
            _xd._o["server-settings"] = {
                _token: "<?php echo esc_html($aime_settings['api_key']) ?>",
                _verbose: true,
                _player_id: "xplayer",
                _player_class: "xplayer",
                _player_parent: "aime-container",
                _theme: "dark",
                _public_folder: "<?php echo esc_html($scripts['aime_public_folder']) ?>",
                _wp_user_id: "string",
                _wp_user_email: "string",
                _wp_site_url: "string",
                _wp_site_name: "string",
                _wp_site_description: "string",
                _wp_site_icon: "string",
                _channel_id: "<?php echo esc_html($aime_settings['channel_id']) ?>",
                _aime_server_url: "<?php echo esc_html($aime_settings['aime_server_url']) ?>"
            }
            _xlog.log("Settings Updated :)");
        </script>

        <div id="floating-div" class="floating-box">
            <h1>Xpell.AI Settings</h1>
            <form id="aime-settings-form" method="post" action="<?php echo esc_html(admin_url('admin-post.php')); ?>">
                <input type="hidden" name="action" value="save_aime_settings">
                <?php wp_nonce_field('aime_settings_nonce', 'aime_settings_nonce_field'); ?>
                <div style="display:grid;gap:10px">

                    <div style="display:grid-column-start:0;">
                        <label for="api_key" style="width:100px;display:inline-block">API Key</label>
                        <input type="text" id="api_key" name="api_key" style="width:250px;display:inline-block" value="<?php echo esc_html($aime_settings['api_key']); ?>">
                    </div>
                    <div>
                        <label for="owner_id" style="width:100px;display:inline-block">Owner ID</label>
                        <input type="text" id="owner_id" name="owner_id" style="width:250px;display:inline-block" value="<?php echo esc_html($aime_settings['owner_id']); ?>">
                    </div>
                    <div style="display:grid-column-start:0;">
                        <label for="aime_server_url" style="width:100px;display:inline-block">Aime Server</label>
                        <input type="text" id="aime_server_url" style="width:250px;display:inline-block" name="aime_server_url" value="<?php echo esc_html($aime_settings['aime_server_url']); ?>">
                    </div>
                    <div style="display:grid-column-start:0;">
                        <label for="first_name" style="width:100px;display:inline-block">First Name</label>
                        <input type="text" id="first_name" style="width:250px;display:inline-block" name="first_name" value="<?php echo esc_html($aime_settings['first_name']); ?>">
                    </div>
                    <div style="display:grid-column-start:0;">
                        <label for="last_name" style="width:100px;display:inline-block">Last Name</label>
                        <input type="text" id="last_name" style="width:250px;display:inline-block" name="last_name" value="<?php echo esc_html($aime_settings['last_name']); ?>">
                    </div>
                    <div style="display:grid-column-start:0;">
                        <label for="channel_id" style="width:100px;display:inline-block">channel id</label>
                        <input type="text" id="channel_id" style="width:250px;display:inline-block" name="channel_id" value="<?php echo esc_html($aime_settings['channel_id']); ?>">

                    </div>

                    <div style="display:grid-column-start:0;">
                        <input type="submit" value="Save">
                    </div>
                </div>
            </form>
        </div>
        <?php
        if (isset($_GET['status']) && $_GET['status'] == 'success') {
            echo '<div class="notice notice-success is-dismissible"><p>Settings saved successfully!</p><p>OK :)</p></div>';
        }
    }

    function handle_aime_settings_form()
    {
       // Verify nonce for security
        // Check if the nonce field is set
        if ( isset($_POST['aime_settings_nonce_field']) ) {
            // Unsure if slashes are present, then sanitize the nonce
            $nonce_field = sanitize_text_field(wp_unslash($_POST['aime_settings_nonce_field'])); // Remove slashes
            // Verify nonce for security
            if ( ! wp_verify_nonce($nonce_field, 'aime_settings_nonce') ) {
                wp_die('Security check failed');
            }
        } else {
            wp_die('Nonce field is missing');
        }

        // Check if the user has the capability to manage options
        if (!current_user_can('manage_options')) {
            wp_die('Unauthorized user');
        }

        // Sanitize and save the API key
        if ( isset($_POST['api_key']) ) {
            $api_key = sanitize_text_field(wp_unslash($_POST['api_key']));
        } 
        if ( isset($_POST['aime_server_url']) ) {
            $aime_server_url = sanitize_text_field(wp_unslash($_POST['aime_server_url']));
        }

        if(isset ($_POST['owner_id'])){
            $owner_id = sanitize_text_field(wp_unslash($_POST['owner_id']));
        }

        if(isset ($_POST['first_name'])){
            $first_name = sanitize_text_field(wp_unslash($_POST['first_name']));
        }

        if(isset ($_POST['last_name'])){
            $last_name = sanitize_text_field(wp_unslash($_POST['last_name']));
        }

        if(isset ($_POST['channel_id'])){
            $channel_id = sanitize_text_field(wp_unslash($_POST['channel_id']));
        }
        
        $aime_settings = array(
            'api_key' => $api_key,
            "aime_server_url" => $aime_server_url,
            'owner_id' => $owner_id,
            'first_name' => $first_name,
            'last_name' => $last_name,
            'channel_id' => $channel_id

        );


        // Update the option in the database
        update_option('aime_settings', $aime_settings);


        // Redirect to the settings page with a success message
        wp_redirect(add_query_arg('status', 'success', admin_url('admin.php?page=xpell-menu-settings')));
        exit();
    }
    // function get_aime_scripts_urls() {
    //     $aime_public_folder = plugin_dir_url(__FILE__) . '../client-lib/aime-wp-js/';

    //     return array(
    //         'aime_public_folder' => $aime_public_folder,
    //         'aime_script' => $aime_public_folder .  'aime-wp-js.es.js',
    //         'xpell_script' => plugin_dir_url(__FILE__) . '../client-lib/xpell/xpell.es.js'
    //     );

    // }

    function aime_connect_menu()
    {

        global $_xwp;


        $scripts = $_xwp->get_aime_scripts_urls();

        // $aime_public_folder = plugin_dir_url(__FILE__) . '../client-lib/aime-wp-js/';
        // $aime_script = $aime_public_folder. 'aime-wp-js.es.js';
        // $xpell_script = plugin_dir_url(__FILE__) . '../client-lib/xpell/xpell.es.js';
        //load settings from wordpress options
        $aime_settings = get_option('aime_settings');
        if (!$aime_settings) {
            $aime_settings = array();
            $aime_settings['api_key'] = false;
            $aime_settings['owner_id'] = '';
            $aime_settings['aime_sever_url'] = $_xwp->$aime_server_default_url;
            $aime_settings['first_name'] = get_user_meta($admin_id, 'first_name', true);
            $aime_settings['last_name'] = get_user_meta($admin_id, 'last_name', true);
            $aime_settings['channel_id'] = '';
        }

        $wp_token = $_xwp->add_aime_nonce('aime_registration_nonce');
        // wp_create_nonce('aime_registration_nonce2');
        // error_log("Nonce Token: menu " . $wp_token);


        $admin_info = get_userdata(1);
        $site_info = get_bloginfo();
        $site_icon = get_site_icon_url();
        $site_url = get_site_url();
        $site_description = get_bloginfo('description');


        ?>
        <div id="aime-container" class="aime-view-container">
        </div>

        <script type="module" defer>
            import {
                start
            } from '<?php echo esc_html($scripts['aime_script']) ?>';

            const serverSettings = {
                _token: "<?php echo esc_html($aime_settings['api_key']) ?>",
                _verbose: true,
                _player_id: "xplayer",
                _player_class: "xplayer",
                _player_parent: "aime-container",
                _theme: "dark",
                _public_folder: "<?php echo esc_html($scripts['aime_public_folder']) ?>",
                _wp_user_id: "<?php echo esc_html($admin_info->ID) ?>",
                _wp_user_email: "<?php echo esc_html($admin_info->user_email) ?>",
                _wp_site_url: "<?php echo esc_html($site_url) ?>",
                _wp_site_name: "<?php echo esc_html($site_info) ?>",
                _wp_site_description: "<?php echo esc_html($site_description) ?>",
                _wp_site_icon: "<?php echo esc_html($site_icon) ?>",
                _wp_token: "<?php echo esc_html($wp_token) ?>",
                _wp_first_name: "<?php echo esc_html($aime_settings['first_name']) ?>",
                _wp_last_name: "<?php echo esc_html($aime_settings['last_name']) ?>",
                _aime_server_url: "<?php echo esc_html($aime_settings['aime_server_url']) ?>",
            }

            // error_log(serverSettings);
            await start(serverSettings); //start the aime-connect app
        </script>
<?php

    }

    function aime_context_menu()
    {

        global $_xwp;


        $scripts = $_xwp->get_aime_scripts_urls();

        // $aime_public_folder = plugin_dir_url(__FILE__) . '../client-lib/aime-wp-js/';
        // $aime_script = $aime_public_folder. 'aime-wp-js.es.js';
        // $xpell_script = plugin_dir_url(__FILE__) . '../client-lib/xpell/xpell.es.js';
        //load settings from wordpress options
        $aime_settings = get_option('aime_settings');
        if (!$aime_settings) {
            $aime_settings = array();
            $aime_settings['api_key'] = false;
            $aime_settings['owner_id'] = '';
            $aime_settings['aime_sever_url'] = $_xwp->$aime_server_default_url;
            $aime_settings['first_name'] = get_user_meta($admin_id, 'first_name', true);
            $aime_settings['last_name'] = get_user_meta($admin_id, 'last_name', true);
            $aime_settings['channel_id'] = '';
        }

        $wp_token = $_xwp->add_aime_nonce('aime_registration_nonce');
        // wp_create_nonce('aime_registration_nonce2');
        // error_log("Nonce Token: menu " . $wp_token);


        $admin_info = get_userdata(1);
        $site_info = get_bloginfo();
        $site_icon = get_site_icon_url();
        $site_url = get_site_url();
        $site_description = get_bloginfo('description');


        ?>
        <div id="aime-container" class="aime-view-container">
        </div>

        <script type="module" defer>
            import {
                start
            } from '<?php echo $scripts['aime_script'] ?>';

            const serverSettings = {
                _token: "<?php echo $aime_settings['api_key'] ?>",
                _verbose: true,
                _player_id: "xplayer",
                _player_class: "xplayer",
                _player_parent: "aime-container",
                _theme: "dark",
                _public_folder: "<?php echo $scripts['aime_public_folder'] ?>",
                _wp_user_id: "<?php echo $admin_info->ID ?>",
                _wp_user_email: "<?php echo $admin_info->user_email ?>",
                _wp_site_url: "<?php echo $site_url ?>",
                _wp_site_name: "<?php echo $site_info ?>",
                _wp_site_description: "<?php echo $site_description ?>",
                _wp_site_icon: "<?php echo $site_icon ?>",
                _wp_token: "<?php echo $wp_token ?>",
                _wp_first_name: "<?php echo $aime_settings['first_name'] ?>",
                _wp_last_name: "<?php echo $aime_settings['last_name'] ?>",
                _aime_server_url: "<?php echo $aime_settings['aime_server_url'] ?>",
                _channel_id: "<?php echo $aime_settings['channel_id'] ?>",
                _aime_script: "context"
            }

            // error_log(serverSettings);
            await start(serverSettings); //start the aime-connect app
        </script>
<?php

    }





    // function create_token()
    // {
    //     $secret = "aime1234";

    //     $payload = array(
    //         "iss" => "http://example.org",
    //         "aud" => "http://example.com",
    //         "iat" => 1356999524,
    //         "nbf" => 1357000000
    //     );

    //     $jwt = JWT::encode($payload, $secret, 'HS256');
    //     return $jwt;
    // }
}

?>